@@ -55,16 +55,13 @@ typedef enum {
 	GDA_CLIENT_SPECS_DROP_DATABASE
 } GdaClientSpecsType;
 
-typedef struct _GdaClientClass   GdaClientClass;
-typedef struct _GdaClientPrivate GdaClientPrivate;
-
 struct _GdaClient {
-	GObject object;
+	GObject           object;
 	GdaClientPrivate *priv;
 };
 
 struct _GdaClientClass {
-	GObjectClass parent_class;
+	GObjectClass      parent_class;
 
 	/* signals */
 	void (* event_notification) (GdaClient *client,
@@ -86,6 +83,7 @@ GdaConnection *gda_client_open_connection                    (GdaClient *client,
 							      const gchar *password,
 							      GdaConnectionOptions options,
 							      GError **error);
+void           gda_client_declare_connection                 (GdaClient *client, GdaConnection *cnc);
 GdaConnection *gda_client_open_connection_from_string        (GdaClient *client,
 							      const gchar *provider_id,
 							      const gchar *cnc_string,
@@ -136,6 +134,8 @@ gboolean       gda_client_begin_transaction                  (GdaClient *client,
 gboolean       gda_client_commit_transaction                 (GdaClient *client, GdaTransaction *xaction);
 gboolean       gda_client_rollback_transaction               (GdaClient *client, GdaTransaction *xaction);
 
+
+
 G_END_DECLS
 
 #endif
