@@ -1,5 +1,5 @@
 /* GDA MySQL provider
- * Copyright (C) 1998 - 2005 The GNOME Foundation.
+ * Copyright (C) 1998 - 2006 The GNOME Foundation.
  *
  * AUTHORS:
  *      Michael Lausch <michael@lausch.at>
@@ -23,21 +23,22 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <libgda/gda-intl.h>
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include "gda-mysql.h"
 #include "gda-mysql-recordset.h"
+#include <libgda/gda-data-model-private.h>
 
 #define PARENT_TYPE GDA_TYPE_DATA_MODEL_ARRAY
 
 struct _GdaMysqlRecordsetPrivate {
-	MYSQL_RES *mysql_res;
-	gint mysql_res_rows;
+	MYSQL_RES     *mysql_res;
+	gint           mysql_res_rows;
 	GdaConnection *cnc;
-	gint ncolumns;
-	gchar *table_name;
-	gboolean row_sync;
+	gint           ncolumns;
+	gchar         *table_name;
+	gboolean       row_sync;
 };
 
 static void gda_mysql_recordset_class_init (GdaMysqlRecordsetClass *klass);
@@ -45,14 +46,14 @@ static void gda_mysql_recordset_init       (GdaMysqlRecordset *recset,
 					    GdaMysqlRecordsetClass *klass);
 static void gda_mysql_recordset_finalize   (GObject *object);
 
-static gint gda_mysql_recordset_get_n_rows			     (GdaDataModelBase *model);
-static GdaColumn *gda_mysql_recordset_describe_column (GdaDataModelBase *model, gint col);
-static const GdaRow *gda_mysql_recordset_get_row		     (GdaDataModelBase *model, gint row);
-static const GdaValue *gda_mysql_recordset_get_value_at		     (GdaDataModelBase *model, gint col, gint row);
-static gboolean gda_mysql_recordset_is_updatable		     (GdaDataModelBase *model);
-static gboolean gda_mysql_recordset_append_row			     (GdaDataModelBase *model, GdaRow *row);
-static gboolean gda_mysql_recordset_remove_row			     (GdaDataModelBase *model, const GdaRow *row);
-static gboolean gda_mysql_recordset_update_row			     (GdaDataModelBase *model, const GdaRow *row);
+static gint gda_mysql_recordset_get_n_rows			     (GdaDataModelRow *model);
+static GdaRow *gda_mysql_recordset_get_row		     (GdaDataModelRow *model, 
+							      gint row, GError **error);
+static const GdaValue *gda_mysql_recordset_get_value_at		     (GdaDataModelRow *model, gint col, gint row);
+static gboolean gda_mysql_recordset_is_updatable		     (GdaDataModelRow *model);
+static gboolean gda_mysql_recordset_append_row			     (GdaDataModelRow *model, GdaRow *row, GError **error);
+static gboolean gda_mysql_recordset_remove_row			     (GdaDataModelRow *model, GdaRow *row, GError **error);
+static gboolean gda_mysql_recordset_update_row			     (GdaDataModelRow *model, GdaRow *row, GError **error);
 
 static GObjectClass *parent_class = NULL;
 
@@ -67,13 +68,12 @@ static void
 gda_mysql_recordset_class_init (GdaMysqlRecordsetClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaDataModelBaseClass *model_class = GDA_DATA_MODEL_BASE_CLASS (klass);
+	GdaDataModelRowClass *model_class = GDA_DATA_MODEL_ROW_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_mysql_recordset_finalize;
 	model_class->get_n_rows = gda_mysql_recordset_get_n_rows;
-	model_class->describe_column = gda_mysql_recordset_describe_column;
 	model_class->get_row = gda_mysql_recordset_get_row;
 	model_class->get_value_at = gda_mysql_recordset_get_value_at;
 	model_class->is_updatable = gda_mysql_recordset_is_updatable;
@@ -161,9 +161,13 @@ fill_gda_value (GdaValue *gda_value, enum enum_field_types type, gchar *value,
 	case FIELD_TYPE_TINY_BLOB :
 	case FIELD_TYPE_MEDIUM_BLOB :
 	case FIELD_TYPE_LONG_BLOB :
-	case FIELD_TYPE_BLOB :
-		gda_value_set_binary (gda_value, value, length);
+	case FIELD_TYPE_BLOB : {
+		GdaBinary bin;
+		bin.data = value;
+		bin.binary_length = length;
+		gda_value_set_binary (gda_value, &bin);
 		break;
+	}
 	case FIELD_TYPE_VAR_STRING :
 	case FIELD_TYPE_STRING :
 		/* FIXME: we might get "[VAR]CHAR(20) BINARY" type with \0 inside
@@ -182,6 +186,8 @@ fill_gda_value (GdaValue *gda_value, enum enum_field_types type, gchar *value,
 		gda_value_set_from_string (gda_value, value, GDA_VALUE_TYPE_TIMESTAMP);
 		break;
 	case FIELD_TYPE_NULL :
+		gda_value_set_null (gda_value);
+		break;
 	case FIELD_TYPE_NEWDATE :
 	case FIELD_TYPE_ENUM :
 	case FIELD_TYPE_SET : /* FIXME */
@@ -205,12 +211,11 @@ fetch_row (GdaMysqlRecordset *recset, gulong rownum)
 	MYSQL_ROW mysql_row;
 	unsigned long *mysql_lengths;
 
-
 	g_return_val_if_fail (GDA_IS_MYSQL_RECORDSET (recset), NULL);
 	g_return_val_if_fail (recset->priv != NULL, 0);
 
 	if (!recset->priv->mysql_res) {
-		gda_connection_add_error_string (recset->priv->cnc, _("Invalid MySQL handle"));
+		gda_connection_add_event_string (recset->priv->cnc, _("Invalid MySQL handle"));
 		return NULL;
 	}
 
@@ -221,7 +226,7 @@ fetch_row (GdaMysqlRecordset *recset, gulong rownum)
 	field_count = mysql_num_fields (recset->priv->mysql_res);
 
 	if (rownum < 0 || rownum >= row_count) {
-		gda_connection_add_error_string (recset->priv->cnc, _("Row number out of range"));
+		gda_connection_add_event_string (recset->priv->cnc, _("Row number out of range"));
 		return NULL;
 	}
 
@@ -250,7 +255,7 @@ fetch_row (GdaMysqlRecordset *recset, gulong rownum)
  */
 
 static gint
-gda_mysql_recordset_get_n_rows (GdaDataModelBase *model)
+gda_mysql_recordset_get_n_rows (GdaDataModelRow *model)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 
@@ -260,89 +265,46 @@ gda_mysql_recordset_get_n_rows (GdaDataModelBase *model)
 	if (recset->priv->row_sync == FALSE)
 		return recset->priv->mysql_res_rows;
 	else
-		return GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_n_rows (model);
+		return GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_n_rows (model);
 }
 
-static GdaColumn *
-gda_mysql_recordset_describe_column (GdaDataModelBase *model, gint col)
-{
-	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
-	GdaMysqlRecordsetPrivate *priv_data;
-	gint field_count;
-	GdaColumn *attrs;
-	MYSQL_FIELD *mysql_field;
-
-	g_return_val_if_fail (GDA_IS_MYSQL_RECORDSET (recset), NULL);
-	g_return_val_if_fail (recset->priv != NULL, 0);
-
-	priv_data = recset->priv;
-
-	if (!priv_data->mysql_res) {
-		gda_connection_add_error_string (priv_data->cnc, _("Invalid MySQL handle"));
-		return NULL;
-	}
-
-	/* create the GdaColumn to be returned */
-	field_count = mysql_num_fields (priv_data->mysql_res);
-	if (col >= field_count)
-		return NULL;
-
-	mysql_field = mysql_fetch_field_direct (priv_data->mysql_res, col);
-	if (!mysql_field)
-		return NULL;
-
-	attrs = gda_column_new ();
-
-	if (mysql_field->name)
-		gda_column_set_name (attrs, mysql_field->name);
-	gda_column_set_defined_size (attrs, mysql_field->length);
-	gda_column_set_table (attrs, mysql_field->table);
-	/* gda_column_set_caption(attrs, ); */
-	gda_column_set_scale (attrs, mysql_field->decimals);
-	gda_column_set_gdatype (attrs, gda_mysql_type_to_gda (mysql_field->type,
-									mysql_field->flags & UNSIGNED_FLAG));
-	gda_column_set_allow_null (attrs, !IS_NOT_NULL (mysql_field->flags));
-	gda_column_set_primary_key (attrs, IS_PRI_KEY (mysql_field->flags));
-	gda_column_set_unique_key (attrs, mysql_field->flags & UNIQUE_KEY_FLAG);
-	/* gda_column_set_references(attrs, ); */
-	gda_column_set_auto_increment (attrs, mysql_field->flags & AUTO_INCREMENT_FLAG);
-	/* attrs->auto_increment_start */
-	/* attrs->auto_increment_step  */
-	gda_column_set_position (attrs, col);
-
-	return attrs;
-}
-
-static const GdaRow *
-gda_mysql_recordset_get_row (GdaDataModelBase *model, gint row)
+static GdaRow *
+gda_mysql_recordset_get_row (GdaDataModelRow *model, gint row, GError **error)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 	GdaMysqlRecordsetPrivate *priv_data;
 	gint fetched_rows;
 	gint i;
-	GdaRow *row_list, *fields = NULL;
+	GdaRow *row_list;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_RECORDSET (recset), NULL);
 	g_return_val_if_fail (recset->priv != NULL, 0);
 
-	row_list = (GdaRow *) GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_row (model, row);
+	row_list = (GdaRow *) GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_row (model, row, 
+										error);
 	if (row_list != NULL)
-		return (const GdaRow *)row_list;	
+		return row_list;	
 
 	priv_data = recset->priv;
 	if (!priv_data->mysql_res) {
-		gda_connection_add_error_string (priv_data->cnc,
+		gda_connection_add_event_string (priv_data->cnc,
 						_("Invalid MySQL handle"));
 		return NULL;
 	}
 	
 	if (row < 0 || row > priv_data->mysql_res_rows) {
-		gda_connection_add_error_string (priv_data->cnc,
-						_("Row number out of range"));
+		gchar *str;
+
+		str = g_strdup_printf (_("Row number out of range 0 - %d"),
+				       priv_data->mysql_res_rows - 1);
+		gda_connection_add_event_string (priv_data->cnc, str);
+		g_set_error (error, 0, 0, str);
+		g_free (str);
+						 
 		return NULL;
 	}
 
-	fetched_rows = GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_n_rows (model);
+	fetched_rows = GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_n_rows (model);
 
 	gda_data_model_freeze (GDA_DATA_MODEL (recset));
 
@@ -351,18 +313,17 @@ gda_mysql_recordset_get_row (GdaDataModelBase *model, gint row)
 		if (!row_list)
 			return NULL;
 
-		if (GDA_DATA_MODEL_BASE_CLASS (parent_class)->append_row (model, row_list) == FALSE)
+		if (! GDA_DATA_MODEL_ROW_CLASS (parent_class)->append_row (model, row_list, NULL))
 			return NULL;
 	}
 
 	gda_data_model_thaw (GDA_DATA_MODEL (recset));
 
-
-	return (const GdaRow *) row_list;
+	return row_list;
 }
 
 static const GdaValue *
-gda_mysql_recordset_get_value_at (GdaDataModelBase *model, gint col, gint row)
+gda_mysql_recordset_get_value_at (GdaDataModelRow *model, gint col, gint row)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 	GdaMysqlRecordsetPrivate *priv_data;
@@ -372,47 +333,49 @@ gda_mysql_recordset_get_value_at (GdaDataModelBase *model, gint col, gint row)
 	g_return_val_if_fail (GDA_IS_MYSQL_RECORDSET (recset), NULL);
 	g_return_val_if_fail (recset->priv != NULL, 0);
 
-	value = GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_value_at (model, col, row);
-	if (value != NULL)
-		return value;
+	if (row < GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_n_rows (model)) {
+		value = GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_value_at (model, col, row);
+		if (value != NULL)
+			return value;
+	}
 
 	priv_data = recset->priv;
         if (!priv_data->mysql_res) {
-		gda_connection_add_error_string (priv_data->cnc,
+		gda_connection_add_event_string (priv_data->cnc,
 						 _("Invalid MySQL handle"));
 		return NULL;
 	}
 
 	if (row < 0 || row > priv_data->mysql_res_rows) {
-		gda_connection_add_error_string (priv_data->cnc,
+		gda_connection_add_event_string (priv_data->cnc,
 						 _("Row number out of range"));
 		return NULL;
 	}
 
 	if (col >= priv_data->ncolumns) {
-		gda_connection_add_error_string (priv_data->cnc,
+		gda_connection_add_event_string (priv_data->cnc,
 						 _("Column number out of range"));
 		return NULL;
 	}	
 
-	fields = gda_mysql_recordset_get_row (model, row);
+	fields = gda_mysql_recordset_get_row (model, row, NULL);
 	return fields != NULL ? gda_row_get_value ((GdaRow *) fields, col) : NULL;
 }
 
 static gboolean
-gda_mysql_recordset_is_updatable (GdaDataModelBase *model)
+gda_mysql_recordset_is_updatable (GdaDataModelRow *model)
 {
 	GdaCommandType cmd_type;
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_RECORDSET (recset), FALSE);
 	
-	cmd_type = gda_data_model_get_command_type (GDA_DATA_MODEL (model));
+	g_object_get (G_OBJECT (model), "command_type", &cmd_type, NULL);
 	return cmd_type == GDA_COMMAND_TYPE_TABLE ? TRUE : FALSE;
 }
 
 static gboolean
-gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
+gda_mysql_recordset_append_row (GdaDataModelRow *model, GdaRow *row, GError **error)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 	GdaMysqlRecordsetPrivate *priv_data;
@@ -432,7 +395,7 @@ gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
 
 	/* checks for valid MySQL handle */
         if (!priv_data->mysql_res) {
-		gda_connection_add_error_string (priv_data->cnc, _("Invalid MySQL handle"));
+		gda_connection_add_event_string (priv_data->cnc, _("Invalid MySQL handle"));
 		return FALSE;
 	}
 
@@ -441,14 +404,14 @@ gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
 	
 	/* checks if the table name has been guessed */
 	if (priv_data->table_name == NULL) {
-		gda_connection_add_error_string (priv_data->cnc,
+		gda_connection_add_event_string (priv_data->cnc,
 						_("Table name could not be guessed"));
 		return FALSE;
 	}
 
 	/* checks for correct number of columns */
 	if (priv_data->ncolumns != gda_row_get_length (row)) {
-		gda_connection_add_error_string (priv_data->cnc,
+		gda_connection_add_event_string (priv_data->cnc,
 						_("Attempt to insert a row with an invalid number of columns"));
 		return FALSE;
 	}
@@ -456,20 +419,20 @@ gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
 	/* check if all results are loaded in array, if not do so */
 	if (priv_data->row_sync == FALSE)
 	{
-		fetched_rows = GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_n_rows (model);
+		fetched_rows = GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_n_rows (model);
 
 		gda_data_model_freeze (GDA_DATA_MODEL (recset));
 
 		for (i = fetched_rows; i < priv_data->mysql_res_rows; i++) {
 			row_list = fetch_row (recset, i);
 			if (!row_list) {
-				gda_connection_add_error_string (priv_data->cnc,
+				gda_connection_add_event_string (priv_data->cnc,
 						_("Can not synchronize array with MySQL result set"));
 				return FALSE;
 			}
 
-			if (GDA_DATA_MODEL_BASE_CLASS (parent_class)->append_row (model, row_list) == FALSE) {
-				gda_connection_add_error_string (priv_data->cnc,
+			if (!GDA_DATA_MODEL_ROW_CLASS (parent_class)->append_row (model, row_list, NULL)) {
+				gda_connection_add_event_string (priv_data->cnc,
 						_("Can not synchronize array with MySQL result set"));
 				return FALSE;
 			}
@@ -520,7 +483,7 @@ gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
 	/* execute append command */
 	rc = mysql_real_query (mysql, sql->str, strlen (sql->str));
 	if (rc != 0) {
-		gda_connection_add_error (
+		gda_connection_add_event (
 			priv_data->cnc, gda_mysql_make_error (mysql));
 		return FALSE;
 	}
@@ -529,8 +492,8 @@ gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
 	g_string_free (sql_value, TRUE);
 
 	/* append row in the array */
-	if (GDA_DATA_MODEL_BASE_CLASS (parent_class)->append_row (model, row) == FALSE) {
-		gda_connection_add_error_string (priv_data->cnc,
+	if (! GDA_DATA_MODEL_ROW_CLASS (parent_class)->append_row (model, row, NULL)) {
+		gda_connection_add_event_string (priv_data->cnc,
 			_("Can not append row to data model"));
 		return FALSE;
 	}
@@ -539,7 +502,7 @@ gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
 }
 
 static gboolean
-gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
+gda_mysql_recordset_remove_row (GdaDataModelRow *model, GdaRow *row, GError **error)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 	GdaMysqlRecordsetPrivate *priv_data;
@@ -560,7 +523,7 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 
 	/* checks for valid MySQL handle */
         if (!priv_data->mysql_res) {
-		gda_connection_add_error_string (priv_data->cnc, _("Invalid MySQL handle"));
+		gda_connection_add_event_string (priv_data->cnc, _("Invalid MySQL handle"));
 		return FALSE;
 	}
 
@@ -569,14 +532,18 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 	
 	/* checks if the given row belongs to the given model */
 	if (gda_row_get_model ((GdaRow *) row) != GDA_DATA_MODEL (model)) {
-		gda_connection_add_error_string (priv_data->cnc,
+		g_set_error (error, 0, 0,
+			     _("Given row doesn't belong to the model."));
+		gda_connection_add_event_string (priv_data->cnc,
 						_("Given row doesn't belong to the model."));
 		return FALSE;
 	}
 
 	/* checks if the table name has been guessed */
 	if (priv_data->table_name == NULL) {
-		gda_connection_add_error_string (priv_data->cnc,
+		g_set_error (error, 0, 0,
+			     _("Table name could not be guessed"));
+		gda_connection_add_event_string (priv_data->cnc,
 						_("Table name could not be guessed"));
 		return FALSE;
 	}
@@ -584,20 +551,22 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 	/* check if all results are loaded in array, if not do so */
 	if (priv_data->row_sync == FALSE)
 	{
-		fetched_rows = GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_n_rows (model);
+		fetched_rows = GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_n_rows (model);
 
 		gda_data_model_freeze (GDA_DATA_MODEL (recset));
 
 		for (i = fetched_rows; i < priv_data->mysql_res_rows; i++) {
 			row_list = fetch_row (recset, i);
 			if (!row_list) {
-				gda_connection_add_error_string (priv_data->cnc,
+				g_set_error (error, 0, 0,
+					     _("Can not synchronize array with MySQL result set"));
+				gda_connection_add_event_string (priv_data->cnc,
 						_("Can not synchronize array with MySQL result set"));
 				return FALSE;
 			}
 
-			if (GDA_DATA_MODEL_BASE_CLASS (parent_class)->append_row (model, row_list) == FALSE) {
-				gda_connection_add_error_string (priv_data->cnc,
+			if (! GDA_DATA_MODEL_ROW_CLASS (parent_class)->append_row (model, row_list, error)) {
+				gda_connection_add_event_string (priv_data->cnc,
 						_("Can not synchronize array with MySQL result set"));
 				return FALSE;
 			}
@@ -628,10 +597,10 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 				priv_data->cnc, 	 
 				gda_row_get_value ((GdaRow *) row, colnum) 	 
      		);
+
 		/* unique column: we will use it as an index */
 		if (gda_column_get_primary_key (attrs) ||
-		    gda_column_get_unique_key (attrs))
-		{
+		    gda_column_get_unique_key (attrs)) {
 			/* fills the 'where' part of the update command */
 			if (colnum != 0)
 				query_where = g_strconcat (query_where, "AND ", NULL);
@@ -646,11 +615,12 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 		}
 
 		g_free (curval);
-		gda_column_free (attrs);
 	}
 
 	if (uk == 0) {
-		gda_connection_add_error_string (priv_data->cnc,
+		g_set_error (error, 0, 0,
+			     _("Model doesn't have at least one unique key."));
+		gda_connection_add_event_string (priv_data->cnc,
 						_("Model doesn't have at least one unique key."));
 		g_free (query_where);
 
@@ -665,7 +635,7 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 	/* execute append command */
 	rc = mysql_real_query (mysql, query, strlen (query));
 	if (rc != 0) {
-		gda_connection_add_error (
+		gda_connection_add_event (
 			priv_data->cnc, gda_mysql_make_error (mysql));
 		g_free (query);
 		g_free (query_where);
@@ -677,8 +647,10 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 	g_free (query_where);
 
 	/* remove row from the array */
-	if (GDA_DATA_MODEL_BASE_CLASS (parent_class)->remove_row (model, row) == FALSE) {
-		gda_connection_add_error_string (priv_data->cnc,
+	if (! GDA_DATA_MODEL_ROW_CLASS (parent_class)->remove_row (model, row, NULL)) {
+		g_set_error (error, 0, 0,
+			     _("Can not remove row from data model"));
+		gda_connection_add_event_string (priv_data->cnc,
 			_("Can not remove row from data model"));
 		return FALSE;
 	}
@@ -687,7 +659,7 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 }
 
 static gboolean
-gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
+gda_mysql_recordset_update_row (GdaDataModelRow *model, GdaRow *row, GError **error)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 	GdaMysqlRecordsetPrivate *priv_data;
@@ -709,7 +681,9 @@ gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 
 	/* checks for valid MySQL handle */
         if (!priv_data->mysql_res) {
-		gda_connection_add_error_string (priv_data->cnc, _("Invalid MySQL handle"));
+		g_set_error (error, 0, 0,
+			     _("Invalid MySQL handle"));
+		gda_connection_add_event_string (priv_data->cnc, _("Invalid MySQL handle"));
 		return FALSE;
 	}
 
@@ -718,14 +692,18 @@ gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 	
 	/* checks if the given row belongs to the given model */
 	if (gda_row_get_model ((GdaRow *) row) != GDA_DATA_MODEL (model)) {
-		gda_connection_add_error_string (priv_data->cnc,
+		g_set_error (error, 0, 0,
+			     _("Given row doesn't belong to the model."));
+		gda_connection_add_event_string (priv_data->cnc,
 						_("Given row doesn't belong to the model."));
 		return FALSE;
 	}
 
 	/* checks if the table name has been guessed */
 	if (priv_data->table_name == NULL) {
-		gda_connection_add_error_string (priv_data->cnc,
+		g_set_error (error, 0, 0,
+			     _("Table name could not be guessed."));
+		gda_connection_add_event_string (priv_data->cnc,
 						_("Table name could not be guessed."));
 		return FALSE;
 	}
@@ -795,13 +773,13 @@ gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 		}
 
 		g_free (newval);
-		gda_column_free (attrs);
 	}
 
 	if (uk == 0) {
-		gda_connection_add_error_string (priv_data->cnc,
+		g_set_error (error, 0, 0,
+			     _("Model does not have at least one non-modified unique key."));	     
+		gda_connection_add_event_string (priv_data->cnc,
 						_("Model does not have at least one non-modified unique key."));
-		g_free (query);
 		g_free (query_set);
 		g_free (query_where);
 
@@ -809,9 +787,10 @@ gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 	}
 
 	if (nuk == 0) {
-		gda_connection_add_error_string (priv_data->cnc,
+		g_set_error (error, 0, 0,
+			     _("Model does not have any non-unique values to update."));
+		gda_connection_add_event_string (priv_data->cnc,
 						_("Model does not have any non-unique values to update."));
-		g_free (query);
 		g_free (query_set);
 		g_free (query_where);
 
@@ -832,14 +811,15 @@ gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 	/* execute update command */
 	rc = mysql_real_query (mysql, query, strlen (query));
 	if (rc != 0) {
-		gda_connection_add_error (
-			priv_data->cnc, gda_mysql_make_error (mysql));
+		GdaConnectionEvent *event = gda_mysql_make_error (mysql);
+		gda_connection_add_event (priv_data->cnc, event);
+		g_set_error (error, 0, 0,
+			     gda_connection_event_get_description (event));
 		return FALSE;
 	}
 	
 	/* emit update signals */
 	gda_data_model_row_updated (GDA_DATA_MODEL (model), gda_row_get_number ((GdaRow *) row));
-	gda_data_model_changed (GDA_DATA_MODEL (model));
 
 	/* clean up */
 	g_free (query);
@@ -889,19 +869,20 @@ gda_mysql_recordset_new (GdaConnection *cnc, MYSQL_RES *mysql_res, MYSQL *mysql)
 	if (mysql_res == NULL) {
 		model->priv->mysql_res_rows = mysql_affected_rows (mysql);
 		return model;
-	} else {
+	} 
+	else 
 		model->priv->mysql_res_rows = mysql_num_rows (model->priv->mysql_res);
-	}
+
 	mysql_fields = mysql_fetch_fields (model->priv->mysql_res);
 	if (mysql_fields != NULL) {
 		gint i;
 
 		model->priv->ncolumns = mysql_num_fields (model->priv->mysql_res);
 		gda_data_model_array_set_n_columns (GDA_DATA_MODEL_ARRAY (model),
-					    model->priv->ncolumns);
+						    model->priv->ncolumns);
 		for (i = 0; i < model->priv->ncolumns; i++) {
-			gda_data_model_set_column_title (GDA_DATA_MODEL (model),
-							 i, mysql_fields[i].name);
+			GdaColumn *column;
+			MYSQL_FIELD *mysql_field;
 
 			/* determine table name */
 			gboolean multi_table = FALSE;
@@ -912,6 +893,22 @@ gda_mysql_recordset_new (GdaConnection *cnc, MYSQL_RES *mysql_res, MYSQL *mysql)
 				model->priv->table_name = g_strdup (mysql_fields[0].table);
 			else
 				model->priv->table_name = NULL;
+
+			/* set GdaColumn attributes */
+			column = gda_data_model_describe_column (GDA_DATA_MODEL (model), i);
+			mysql_field = &(mysql_fields[i]);
+			gda_column_set_title (column, mysql_field->name);
+			if (mysql_field->name)
+				gda_column_set_name (column, mysql_field->name);
+			gda_column_set_defined_size (column, mysql_field->length);
+			gda_column_set_table (column, mysql_field->table);
+			gda_column_set_scale (column, mysql_field->decimals);
+			gda_column_set_gda_type (column, gda_mysql_type_to_gda (mysql_field->type,
+									       mysql_field->flags & UNSIGNED_FLAG));
+			gda_column_set_allow_null (column, !IS_NOT_NULL (mysql_field->flags));
+			gda_column_set_primary_key (column, IS_PRI_KEY (mysql_field->flags));
+			gda_column_set_unique_key (column, mysql_field->flags & UNIQUE_KEY_FLAG);
+			gda_column_set_auto_increment (column, mysql_field->flags & AUTO_INCREMENT_FLAG);
 		}
 	}
 
