@@ -1,9 +1,10 @@
 /* GDA Sybase provider
- * Copyright (C) 1998-2002 The GNOME Foundation.
+ * Copyright (C) 1998 - 2005 The GNOME Foundation.
  *
  * AUTHORS:
  *         Mike Wingert <wingert.3@postbox.acs.ohio-state.edu>
  *         Holger Thon <holger.thon@gnome-db.org>
+ *         Vivien Malerba <malerba@gnome-db.org>
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -22,9 +23,10 @@
  */
 
 #if defined(HAVE_CONFIG_H)
+#  include <config.h>
 #endif
 
-#include <libgda/gda-intl.h>
+#include <glib/gi18n-lib.h>
 #include <libgda/gda-data-model.h>
 #include <string.h>
 #include <ctpublic.h>
@@ -37,7 +39,7 @@
 #  undef PARENT_TYPE
 #endif
 
-#define PARENT_TYPE GDA_TYPE_DATA_MODEL
+#define PARENT_TYPE GDA_TYPE_DATA_MODEL_ROW
 
 static GObjectClass *parent_class = NULL;
 
@@ -46,68 +48,57 @@ static void gda_sybase_recordset_init (GdaSybaseRecordset *recset,
                                        GdaSybaseRecordsetClass *klass);
 static void gda_sybase_recordset_finalize (GObject *object);
 
-static GdaFieldAttributes *gda_sybase_recordset_describe_column (GdaDataModel *model,
-                                                                 gint col);
-static gint gda_sybase_recordset_get_n_rows (GdaDataModel *model);
-static gint gda_sybase_recordset_get_n_columns (GdaDataModel *model);
-static const GdaRow *gda_sybase_recordset_get_row (GdaDataModel *model,
-                                                   gint row);
-static const GdaValue *gda_sybase_recordset_get_value_at (GdaDataModel *model,
+static void gda_sybase_recordset_describe_column (GdaDataModel *model, gint col);
+static gint gda_sybase_recordset_get_n_rows (GdaDataModelRow *model);
+static gint gda_sybase_recordset_get_n_columns (GdaDataModelRow *model);
+static const GdaRow *gda_sybase_recordset_get_row (GdaDataModelRow *model,
+                                                   gint row, GError **error);
+static const GdaValue *gda_sybase_recordset_get_value_at (GdaDataModelRow *model,
                                                           gint col,
                                                           gint row);
 
-static GdaFieldAttributes *
+static void
 gda_sybase_recordset_describe_column (GdaDataModel *model, gint col)
 {
 	GdaSybaseRecordset *recset = (GdaSybaseRecordset *) model;
 	CS_DATAFMT         *colinfo = NULL;
-	GdaFieldAttributes *attribs = NULL;
+	GdaColumn          *attribs = NULL;
 	gchar              name[256];
 
-	g_return_val_if_fail (GDA_IS_SYBASE_RECORDSET (recset), NULL);
-	g_return_val_if_fail (recset->priv != NULL, NULL);
-	g_return_val_if_fail (recset->priv->columns != NULL, NULL);
+	g_return_if_fail (GDA_IS_SYBASE_RECORDSET (recset));
+	g_return_if_fail (recset->priv != NULL);
+	g_return_if_fail (recset->priv->columns != NULL);
+	g_return_if_fail (col < recset->priv->columns->len);
 
-	if (col >= recset->priv->columns->len) {
-		return NULL;
-	}
 	colinfo = g_ptr_array_index (recset->priv->columns, col);
+	g_return_if_fail (colinfo);
 
-	if (!colinfo) {
-		return NULL;
-	}
-
-	attribs = gda_field_attributes_new ();
-
-	if (!attribs) {
-		return NULL;
-	}
+	attribs = gda_data_model_describe_column (model, col);
 
 	memcpy (name, colinfo->name,
 	        colinfo->namelen);
-	//name[colinfo->namelen + 1] = '\0';
+	name[colinfo->namelen + 1] = 0
 
-	gda_field_attributes_set_name (attribs, name);
-	gda_field_attributes_set_scale (attribs, colinfo->scale);
-	gda_field_attributes_set_gdatype (attribs,
-	         gda_sybase_get_value_type (colinfo->datatype));
-	gda_field_attributes_set_defined_size (attribs, colinfo->maxlength);
+		gda_column_set_title (attribs, name);
+	gda_column_set_name (attribs, name);
+	gda_column_set_scale (attribs, colinfo->scale);
+	gda_column_set_gda_type (attribs,
+				gda_sybase_get_value_type (colinfo->datatype));
+	gda_column_set_defined_size (attribs, colinfo->maxlength);
 
 	// FIXME:
-	gda_field_attributes_set_references (attribs, "");
-	gda_field_attributes_set_primary_key (attribs, FALSE);
-	//gda_field_attributes_set_primary_key (attribs,
+	gda_column_set_references (attribs, "");
+	gda_column_set_primary_key (attribs, FALSE);
+	//gda_column_set_primary_key (attribs,
 	//         (colinfo->status & CS_KEY) == CS_KEY);
-	gda_field_attributes_set_unique_key (attribs, FALSE);
+	gda_column_set_unique_key (attribs, FALSE);
 	
-	gda_field_attributes_set_allow_null (attribs,
-	         (colinfo->status & CS_CANBENULL) == CS_CANBENULL);
-	
-	return attribs;
+	gda_column_set_allow_null (attribs,
+				   (colinfo->status & CS_CANBENULL) == CS_CANBENULL);
 }
 
 static gint
-gda_sybase_recordset_get_n_rows (GdaDataModel *model)
+gda_sybase_recordset_get_n_rows (GdaDataModelRow *model)
 {
 	GdaSybaseRecordset *recset = (GdaSybaseRecordset *) model;
 
@@ -116,7 +107,7 @@ gda_sybase_recordset_get_n_rows (GdaDataModel *model)
 }
 
 static gint
-gda_sybase_recordset_get_n_columns (GdaDataModel *model)
+gda_sybase_recordset_get_n_columns (GdaDataModelRow *model)
 {
 	GdaSybaseRecordset *recset = (GdaSybaseRecordset *) model;
 
@@ -125,7 +116,7 @@ gda_sybase_recordset_get_n_columns (GdaDataModel *model)
 }
 
 static const GdaRow *
-gda_sybase_recordset_get_row (GdaDataModel *model, gint row)
+gda_sybase_recordset_get_row (GdaDataModelRow *model, gint row, GError **error)
 {
 	GdaSybaseRecordset *recset = (GdaSybaseRecordset *) model;
 
@@ -142,7 +133,7 @@ gda_sybase_recordset_get_row (GdaDataModel *model, gint row)
 }
 
 static const GdaValue *
-gda_sybase_recordset_get_value_at (GdaDataModel *model, gint col, gint row)
+gda_sybase_recordset_get_value_at (GdaDataModelRow *model, gint col, gint row)
 {
 	GdaSybaseRecordset *recset = (GdaSybaseRecordset *) model;
 	const GdaRow *fields;
@@ -153,7 +144,7 @@ gda_sybase_recordset_get_value_at (GdaDataModel *model, gint col, gint row)
 	if (col >= recset->priv->colcnt)
 		return NULL;
 
-	fields = gda_sybase_recordset_get_row (model, row);
+	fields = gda_sybase_recordset_get_row (model, row, NULL);
 	return fields != NULL ? gda_row_get_value ((GdaRow *)fields, col) : NULL;
 }
 
@@ -161,12 +152,11 @@ static void
 gda_sybase_recordset_class_init (GdaSybaseRecordsetClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaDataModelClass *model_class = GDA_DATA_MODEL_CLASS (klass);
+	GdaDataModelRowClass *model_class = GDA_DATA_MODEL_ROW_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_sybase_recordset_finalize;
-	model_class->describe_column = gda_sybase_recordset_describe_column;
 	model_class->get_n_rows = gda_sybase_recordset_get_n_rows;
 	model_class->get_n_columns = gda_sybase_recordset_get_n_columns;
 	model_class->get_row = gda_sybase_recordset_get_row;
@@ -198,7 +188,7 @@ gda_sybase_recordset_finalize (GObject *object)
 				GdaRow *row = (GdaRow *) g_ptr_array_index (recset->priv->rows, 0);
 
 				if (row != NULL) {
-					gda_row_free (row);
+					g_object_unref (row);
 					row = NULL;
 				}
 				g_ptr_array_remove_index (recset->priv->rows, 0);
@@ -240,7 +230,7 @@ gda_sybase_create_current_row (GdaSybaseRecordset *recset)
 	g_return_val_if_fail (recset->priv != NULL, NULL);
 	g_return_val_if_fail (recset->priv != NULL, NULL);
 
-	row = gda_row_new (recset->priv->columns->len);
+	row = gda_row_new (GDA_DATA_MODEL(recset),recset->priv->columns->len);
 	g_return_val_if_fail (row != NULL, NULL);
 
 	for (i = 0; i < recset->priv->columns->len; i++) {
@@ -280,7 +270,7 @@ gda_sybase_recordset_get_type (void)
 	return type;
 }
 
-static GdaSybaseRecordset *
+GdaSybaseRecordset *
 gda_sybase_process_row_result (GdaConnection           *cnc,
                                GdaSybaseConnectionData *scnc,
                                gboolean                fetchall)
@@ -288,7 +278,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 	GdaSybaseRecordset *srecset = NULL;
 	GdaSybaseField     *sfield = NULL;
 	GdaRow             *row = NULL;
-	GdaError           *error = NULL;
+	GdaConnectionEvent *error = NULL;
 	gboolean           columns_set = FALSE;
 	gboolean           sfields_allocated = TRUE;
 	gint               i = 0;
@@ -306,7 +296,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		srecset->priv->scnc = scnc;
 	} else {
 		// most probably a lack of ram failure, 
-		// so don't alloc e.g. an instance of GdaError
+		// so don't alloc e.g. an instance of GdaConnectionEvent
 		if (srecset) {
 			g_object_unref (srecset);
 			srecset = NULL;
@@ -315,6 +305,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		scnc->ret = ct_cancel (NULL, scnc->cmd, CS_CANCEL_CURRENT);
 		if (scnc->ret != CS_SUCCEED) {
 			sybase_error_msg (_("ct_cancel() failed."));
+			sybase_check_messages(cnc);
 		}
 		return NULL;
 	}
@@ -328,14 +319,15 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		error = gda_sybase_make_error (scnc,
 		                               _("%s failed while processing a row result."),
 		                               "ct_res_info()");
-		gda_connection_add_error (cnc, error);
+		gda_connection_add_event (cnc, error);
+		sybase_check_messages(cnc);
 		return NULL;
 	}
 	if (srecset->priv->colcnt <= 0) {
 		error = gda_sybase_make_error (scnc,
 		                               _("%s returned <= 0 columns."),
 		                               "ct_res_info()");
-		gda_connection_add_error (cnc, error);
+		gda_connection_add_event (cnc, error);
 		return NULL;
 	}
 
@@ -359,6 +351,7 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		if (scnc->ret != CS_SUCCEED) {
 			sybase_error_msg (_("Could not call %s while processing row resultset."),
 			                  "ct_cancel()");
+			sybase_check_messages(cnc);
 		}
 		return srecset;
 	}
@@ -376,7 +369,8 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		if (scnc->ret != CS_SUCCEED) {
 			error = gda_sybase_make_error (scnc, 
 			                               _("Could not run %s on column %d"), "ct_describe()", i);
-			gda_connection_add_error (cnc, error);
+			gda_connection_add_event (cnc, error);
+			sybase_check_messages(cnc);
 			break;
 		}
 
@@ -384,7 +378,8 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		if (sfield->data == NULL) {
 			error = gda_sybase_make_error (scnc,
 			                               _("Could not allocate data placeholder for column %d"), i);
-			gda_connection_add_error (cnc, error);
+			gda_connection_add_event (cnc, error);
+			sybase_check_messages(cnc);
 			break;
 		}
 
@@ -395,14 +390,13 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		if (scnc->ret != CS_SUCCEED) {
 			error = gda_sybase_make_error (scnc,
 			                               _("Could not run %s on column %d"), "ct_bind()", i);
-			gda_connection_add_error (cnc, error);
+			gda_connection_add_event (cnc, error);
+			sybase_check_messages(cnc);
 			break;
 		}
 
-		if (sfield->fmt.namelen > 0) {
-			gda_data_model_set_column_title (GDA_DATA_MODEL (srecset),
-			                                 i, sfield->fmt.name);
-		}
+		if (sfield->fmt.namelen > 0) 
+			gda_sybase_recordset_describe_column (GDA_DATA_MODEL (srecset), i);
 	}
 
 	// catch error: ct_describe or g_new0 or ct_bind failed
@@ -413,7 +407,8 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		scnc->ret = ct_cancel (NULL, scnc->cmd, CS_CANCEL_ALL);
 		if (scnc->ret != CS_SUCCEED) {
 			error = gda_sybase_make_error (scnc, _("Could not run %s to cancel processing row resultset."), "ct_cancel");
-			gda_connection_add_error (cnc, error);
+			gda_connection_add_event (cnc, error);
+			sybase_check_messages(cnc);
 		}
 		
 		return srecset;
@@ -428,7 +423,8 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 		
 		if (scnc->ret == CS_ROW_FAIL) {
 			error = gda_sybase_make_error (scnc, _("%s failed on row %d"), "ct_fetch()", row_cnt);
-			gda_connection_add_error (cnc, error);
+			gda_connection_add_event (cnc, error);
+			sybase_check_messages(cnc);
 		}
 		
 		// srecset->columns contains the data;
@@ -443,97 +439,150 @@ gda_sybase_process_row_result (GdaConnection           *cnc,
 //	sybase_debug_msg (_("Got %d of %d rows"), srecset->priv->rowcnt, row_cnt);
 	// step 5: verify ct_fetch's last return code
 	switch (scnc->ret) {
-		case CS_END_DATA:
+	case CS_END_DATA:
 //			sybase_debug_msg (_("Row processing succeeded."));
-			break;
-		case CS_CANCELED:
+		break;
+	case CS_CANCELED:
 //			sybase_debug_msg (_("Row processing canceled."));
-			break;
-		case CS_FAIL:
+		break;
+	case CS_FAIL:
 //			sybase_debug_msg (_("%s returned CS_FAIL. Probably o.k."), "ct_fetch()");
-			break;
-		default:
-			error = gda_sybase_make_error (scnc, _("%s terminated with unexpected return code."), "ct_fetch()");
-			gda_connection_add_error (cnc, error);
+		break;
+	default:
+		error = gda_sybase_make_error (scnc, _("%s terminated with unexpected return code."), "ct_fetch()");
+		gda_connection_add_event (cnc, error);
+		sybase_check_messages(cnc);
 	}
 	
 	return srecset;
 }
 
-// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-// gda_sybase_recordset_new is a general result-processing routine
-// for processing a single ct_results() step.
-// 
-// Therefore it may also return no result, e.g. if the resulttype is
-// CS_CMD_DONE, i.e. we finished processing a command.
-// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-GdaDataModel *
-gda_sybase_recordset_new (GdaConnection *cnc,
-                          gboolean fetchall)
+
+GdaSybaseRecordset *
+gda_sybase_process_msg_result (GdaConnection *cnc,
+			       GdaSybaseConnectionData *scnc)
 {
-	GdaSybaseConnectionData *scnc = NULL;
-	GdaSybaseRecordset      *srecset = NULL;
-//	GdaSybaseField          *sfield = NULL;
-//	GdaRow                  *row = NULL;
-	GdaError                *error = NULL;
-//	gboolean                columns_set = FALSE;
-//	gint                    i = 0;
-
-	g_return_val_if_fail (GDA_IS_CONNECTION(cnc), NULL);
-	scnc = (GdaSybaseConnectionData *) g_object_get_data (G_OBJECT (cnc),
-	                                           OBJECT_DATA_SYBASE_HANDLE);
-	g_return_val_if_fail (scnc != NULL, NULL);
-	g_return_val_if_fail (scnc->cmd != NULL, NULL);
-
-	// now nothing may fail;
-	// a result not processed may block the entire connection
-	// each process_* routine must check if recset is valid,
-	// otherwise alt. a) too process the results (eventually discard)
-	//           alt. b) call ct_cancel, dropping _all_ results of
-	//                   the entire query
 	
-	
-	// we are within the resultset processing loop,
-	// ct_results (scnc->cmd, &scnc->res_type) = CS_SUCCEED
+	// in ASE when a stored procedure uses the print statement, it 
+	// generates a CS_MSG_RESULT result code, treat it like another result set.
 
-	switch (scnc->res_type) {
-		case CS_ROW_RESULT:
-			srecset = gda_sybase_process_row_result (cnc, scnc,
-			                                         TRUE);
-			break;
-	// FIXME: implement CS_CURSOR_RESULT .. CS_COMPUTEFMT_RESULT
-		case CS_CURSOR_RESULT:
-		case CS_PARAM_RESULT:
-		case CS_STATUS_RESULT:
-		case CS_COMPUTE_RESULT:
-		case CS_MSG_RESULT:
-		case CS_DESCRIBE_RESULT:
-		case CS_ROWFMT_RESULT:
-		case CS_COMPUTEFMT_RESULT:
-			scnc->ret = ct_cancel (NULL, scnc->cmd,
-			                       CS_CANCEL_CURRENT);
-			break;
-		case CS_CMD_DONE:
-//			sybase_debug_msg (_("Done with resultset."));
-			break;
-		case CS_CMD_SUCCEED:
-//			sybase_debug_msg (_("Command returned no rows."));
-			srecset = g_object_new (GDA_TYPE_SYBASE_RECORDSET, NULL);
-			if ((srecset != NULL) && (srecset->priv != NULL)
-			    && (srecset->priv->columns != NULL) 
-			    && (srecset->priv->rows != NULL)) {
-				srecset->priv->cnc = cnc;
-				srecset->priv->scnc = scnc;
-			}
-			break;
-		case CS_CMD_FAIL:
-			sybase_debug_msg (_("%s returned %s"),
-			                  "ct_results()", "CS_CMD_FAIL");
-			break;
+	GdaSybaseRecordset *srecset = NULL;
+	GdaSybaseField *sfield = NULL;
+	//	gboolean sfields_allocated = TRUE;
+	GdaRow *row = NULL;
+	GdaValue *val = NULL;
+	gchar *message;
+	CS_INT msgcnt = 0;
+	GdaConnectionEvent *error;
+	CS_SERVERMSG msg; 
+	CS_RETCODE ret;
+
+	// called within gda_sybase_recordset_new
+	srecset = g_object_new (GDA_TYPE_SYBASE_RECORDSET, NULL);
+
+	srecset->priv->cnc = cnc;
+	srecset->priv->scnc = scnc;
+
+	// see if there is a message
+	ret = ct_diag (scnc->connection,
+		       CS_STATUS, 
+		       CS_ALLMSG_TYPE, 
+		       CS_UNUSED,
+		       &msgcnt);
+	
+	if ( ret != CS_SUCCEED ) {
+		error = gda_connection_event_new();
+		g_return_val_if_fail (error != NULL, FALSE);
+		gda_connection_event_set_description (error, 
+						      _("An error occurred when attempting to retrieve a server message count for resultset"));
+		gda_connection_event_set_code (error, -1);
+		gda_connection_event_set_source (error, "gda-sybase");
+		gda_connection_event_set_sqlstate (error, _("Not available"));					
+		gda_connection_add_event (cnc, error);		
+		return NULL;
 	}
 
-	if (srecset) 
-		return GDA_DATA_MODEL (srecset);
-	else
+	if ( msgcnt < 1 ){
+		sybase_debug_msg (_("attempting to make recordset and msg count != 1 !"));
 		return NULL;
+	}
+
+	ret = ct_diag (scnc->connection, 
+		       CS_GET, 
+		       CS_SERVERMSG_TYPE, 
+		       1, 
+		       &msg); 
+	if ( ret != CS_SUCCEED ) { 
+		error = gda_connection_event_new();
+		g_return_val_if_fail (error != NULL, FALSE);
+
+		gda_connection_event_set_description (error,
+						      _("An error occurred when attempting to retrieve a server message for recordset"));
+		gda_connection_event_set_code (error, -1);
+		gda_connection_event_set_source (error, "gda-sybase");
+		gda_connection_event_set_sqlstate (error, _("Not available"));					
+		gda_connection_add_event (cnc, error);
+		return NULL;					
+	} 	
+
+	srecset->priv->colcnt = 1;
+
+	sfield = g_new0 (GdaSybaseField, 1);
+
+	if (sfield != NULL) {
+		g_ptr_array_add (srecset->priv->columns, sfield);
+	} 
+	else {
+		g_object_unref (srecset);
+		srecset = NULL;
+		sybase_error_msg (_("Could not allocate structure for column metainformation."));
+		
+		scnc->ret = ct_cancel (NULL, scnc->cmd, CS_CANCEL_ALL);
+		if (scnc->ret != CS_SUCCEED) {
+			sybase_error_msg (_("Could not call %s while processing row resultset."),
+			                  "ct_cancel()");
+			sybase_check_messages(cnc);
+		}
+		return srecset;
+	}		
+
+	memset (&sfield->fmt, 0, sizeof(sfield->fmt));
+	sfield->fmt.namelen = 0;
+	sfield->fmt.datatype = CS_CHAR_TYPE;
+	sfield->fmt.scale = 0;
+	sfield->fmt.precision = 0;
+	sfield->fmt.status = CS_CANBENULL;
+	sfield->fmt.count = 1;
+	sfield->fmt.locale = NULL;
+
+	row = gda_row_new(GDA_DATA_MODEL(srecset), 1);
+	val = gda_row_get_value(row,0);
+
+	message = g_strdup_printf("%s", (msg.text) ? msg.text : "");
+	sfield->fmt.maxlength = strlen(message);
+
+	srecset->priv->rowcnt = 1;	
+	gda_value_set_string(val,(gchar *)message);
+	g_ptr_array_add(srecset->priv->rows, row);
+
+	// clear the message so we don't get bugged by it later
+	ret = ct_diag (scnc->connection, 
+		       CS_CLEAR, 
+		       CS_SERVERMSG_TYPE, 
+		       CS_UNUSED, 
+		       NULL);   
+	if ( ret != CS_SUCCEED ) { 
+		error = gda_connection_event_new();
+		g_return_val_if_fail (error != NULL, FALSE);
+		gda_connection_event_set_description (error, 
+						      _("call to ct_diag failed when attempting to clear the server messages"));
+		gda_connection_event_set_code (error, -1);
+		gda_connection_event_set_source (error, "gda-sybase");
+		gda_connection_event_set_sqlstate (error, _("Not available"));					
+		gda_connection_add_event (cnc, error);
+		return NULL;
+	} 	
+
+	return srecset;
 }
+																														
